#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 23.04.12
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Befehle, die eine Benutzerinteraktion notwendig machen oder
## nicht erwnschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausfhren zu knnen, mssen die Pakete
## coin, FSA, irr, vcd
## samt Abhngigkeiten installiert werden, vgl. Abschnitt 1.2.7:
##
## install.packages(c("coin", "FSA", "irr", "vcd"),
##                  repos=c("http://www.rforge.net/", "http://ftp5.gwdg.de/pub/misc/cran/"))
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 6 Methoden der Biostatistik
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 6.1 Anpassungstests
##### 6.1.1
draws <- 7
hits  <- 5
pH0   <- 0.25
binom.test(hits, draws, p=pH0, alternative="greater", conf.level=0.95)

binom.test(10, 20, p=0.25, alternative="two.sided")
sum(dbinom(10:20, 20, 0.25)) + sum(dbinom(0, 20, 0.25))

(pVal  <- 1-pbinom(hits-1, draws, pH0))
(critB <- qbinom(1-0.05, draws, pH0))

resG <- binom.test(hits, draws, p=pH0, alternative="greater")
resG$p.value
resL <- binom.test(draws-hits, draws, p=pH0, alternative="less")
resL$p.value
resG$p.value + resL$p.value

##### 6.1.2
queue <- c("f", "m", "m", "f", "m", "f", "f", "f")
Ns    <- table(queue)
(runs <- rle(queue))
(rr   <- length(runs$lengths))
(rr1  <- sum(runs$values == names(Ns)[1]))
(rr2  <- sum(runs$values == names(Ns)[2]))

getP <- function(r1, r2, n1, n2) {
    # Anzahl Iterationen einer Gruppe hchstens Gruppengre
    if((r1 > n1) | (r2 > n2)) { return(0) }
    
    # Punktwahrscheinlichkeit fr r1+r2 ungerade
    p <- (choose(n1-1, r1-1) * choose(n2-1, r2-1)) / choose(n1+n2, n1)
    
    # Punktwahrscheinlichkeit fr r1+r2 gerade: das doppelte von ungerade
    if(((r1+r2) %% 2) == 0) { 2*p } else { p }
}

nn1   <- Ns[1]
nn2   <- Ns[2]
N     <- sum(Ns)
rMin  <- 2
(rMax <- ifelse(nn1 == nn2, N, 2*min(nn1, nn2) + 1))

p3.2 <- getP(3, 2, nn1, nn2)
p2.3 <- getP(2, 3, nn1, nn2)
p3.3 <- getP(3, 3, nn1, nn2)
p4.3 <- getP(4, 3, nn1, nn2)
p3.4 <- getP(3, 4, nn1, nn2)

(pValGr <- p3.2 + p2.3 + p3.3 + p4.3 + p3.4)

p2.2 <- getP(2, 2, nn1, nn2)
p1.2 <- getP(1, 2, nn1, nn2)
p2.1 <- getP(2, 1, nn1, nn2)
p1.1 <- getP(1, 1, nn1, nn2)

(pValLe <- p2.2 + p1.2 + p2.1 + p1.1)

(pValGr + pValLe)

muR   <- 1 + ((2*nn1*nn2) / N)
varR  <- (2*nn1*nn2*(2*nn1*nn2 - N)) / (N^2 * (N-1))
rZ    <- (rr-muR) / sqrt(varR)
(pVal <- 1-pnorm(rZ))

##### 6.1.3
vec <- rnorm(8, mean=1, sd=2)
## fr Daten des Buches diese Datei laden:
# load("data_ks.Rdata")
ks.test(vec, "pnorm", mean=1, sd=2, alternative="two.sided")

myStep  <- ecdf(vec)
sortVec <- sort(vec)
emp     <- myStep(sortVec)
theo    <- pnorm(sortVec, mean=1, sd=2)

diff1   <- emp-theo
diff2   <- c(0, emp[1:(length(emp)-1)]) - theo

(DtwoS  <- max(abs(c(diff1, diff2))))
(Dless  <- abs(min(c(diff1, diff2))))
(Dgreat <- abs(max(c(diff1, diff2))))

ks.test(vec, "pnorm", mean=1, sd=2, alternative="less")$statistic
ks.test(vec, "pnorm", mean=1, sd=2, alternative="greater")$statistic

dev.new(width=7, height=7)
myX <- seq(-6, 6, length.out=200)
plot(myStep, main="Kolmogorov-Smirnov-Anpassungstest", xlab=NA, cex.lab=1.4)
points(myX, pnorm(myX, mean=1, sd=2), type="l")
matlines(rbind(sortVec, sortVec), rbind(emp, theo),
         col=rgb(0, 0, 1, 0.7), lty=1, lwd=2)
matlines(rbind(sortVec, sortVec), rbind(c(0, emp[1:(length(emp)-1)]), theo),
         col=rgb(1, 0, 0, 0.5), lty=1, lwd=2)
legend(x="topleft", legend=c("direkte Differenzen", "verschobene Differenzen"),
       col=c("blue", "red"), lwd=2)

##### 6.1.4
nToss  <- 50
nCateg <- 6
pH0    <- rep(1/nCateg, nCateg)
myData <- sample(1:nCateg, nToss, replace=TRUE)
## fr Daten des Buches diese Datei laden:
# load("data_chisqDistrFix.Rdata")
(tab   <- table(myData))
chisq.test(tab, p=pH0)

expected   <- pH0 * nToss
(statChisq <- sum((tab-expected)^2 / expected))
(pVal      <- 1-pchisq(statChisq, nCateg-1))

##### 6.1.5
myData    <- rnorm(100, mean=100, sd=15)
## fr Daten des Buches diese Datei laden:
# load("data_chisqDistrClass.Rdata")
limits    <- c(80, 90, 100, 110, 120)
dataCut   <- cut(myData, c(-Inf, limits, Inf))
(dataFreq <- table(dataCut))

zLimits   <- (limits-mean(myData)) / sd(myData)
zLimits   <- c(-Inf, zLimits, Inf)
(freqExp  <- diff(pnorm(zLimits)) * length(myData))
(resChisq <- chisq.test(dataFreq, p=diff(pnorm(zLimits))))

statChisq <- resChisq$statistic
realDf    <- resChisq$parameter - 2
(realPval <- 1-pchisq(statChisq, realDf))

#####---------------------------------------------------------------------------
##### 6.2 Analyse von (gemeinsamen) Hufigkeiten kategorialer Variablen
##### 6.2.1
smokes   <- factor(sample(c("no", "yes"), 50, replace=TRUE))
siblings <- factor(round(abs(rnorm(50, 1, 0.5))))
## fr Daten des Buches diese Datei laden:
# load("data_chisqInd.Rdata")
cTab     <- table(smokes, siblings)
addmargins(cTab)
chisq.test(cTab)

J          <- nlevels(smokes)
K          <- nlevels(siblings)
expected   <- outer(rowSums(cTab), colSums(cTab)) / sum(cTab)
(statChisq <- sum((cTab-expected)^2 / expected))
(pVal      <- 1-pchisq(statChisq, (J-1)*(K-1)))

##### 6.2.2
voteX   <- rep(LETTERS[1:5], c(3,  8, 12, 19, 8))
voteY   <- rep(LETTERS[1:5], c(8, 17, 16,  7, 2))
vote    <- c(voteX, voteY)
studies <- factor(rep(c("X", "Y"), c(length(voteX), length(voteY))))
cTab    <- table(studies, vote)
addmargins(cTab)
chisq.test(cTab, simulate.p.value=TRUE)

##### 6.2.3
total <- c(5000, 5000, 5000)
hits  <- c(585, 610, 539)
prop.test(hits, total)

(mat <- cbind(hits, total-hits))
chisq.test(mat)

##### 6.2.4
disease <- factor(rep(c("no", "yes"),   c(10, 5)))
diagN   <- rep(c("isHealthy", "isIll"), c( 8, 2))
diagY   <- rep(c("isHealthy", "isIll"), c( 1, 4))
diagT   <- factor(c(diagN, diagY))
contT1  <- table(disease, diagT)
addmargins(contT1)
fisher.test(contT1, alternative="greater")

(p1 <- choose(8+1, 8) * choose(6, 2) / choose(15, 10))
dhyper(8, 10, 5, 9)
(p2 <- choose(9+0, 9) * choose(6, 1) / choose(15, 10))
dhyper(9, 10, 5, 9)
(pVal <- p1+p2)

##### 6.2.5
smokesFem  <- rbinom(20, size=1, p=0.6)
smokesMale <- rbinom(20, size=1, p=0.4)
smokes     <- factor(c(smokesFem, smokesMale), labels=c("no", "yes"))
sex        <- factor(rep(c("f", "m"), c(20, 20)))
## fr Daten des Buches diese Datei laden:
# load("data_fisherDistrEq.Rdata")
contT2     <- table(sex, smokes)
addmargins(contT2)
fisher.test(contT2, alternative="less")

##### 6.2.6
## fr Daten des Buches diese Datei laden:
# load("data_fisherInd.Rdata")
addmargins(contT1)
TP <- contT1[2, 2]
TN <- contT1[1, 1]
FP <- contT1[1, 2]
FN <- contT1[2, 1]

(prevalence  <- sum(contT1[2, ]) / sum(contT1))
(sensitivity <- recall    <- TP / (TP+FN))
(specificity <- TN / (TN+FP))
(relevance   <- precision <- TP / (TP+FP))
(probT       <- sum(diag(contT1)) / sum(contT1))
(Fval        <- 2 * (precision*recall) / (precision+recall))

#####---------------------------------------------------------------------------
##### 6.3 Mae fr Zusammenhang und bereinstimmung kategorialer Daten
##### 6.3.1
vec1 <- c(100, 76, 56, 99, 50, 62, 36, 69, 55,  17)
vec2 <- c(42,  74, 22, 99, 73, 44, 10, 68, 19, -34)
cor(vec1, vec2, method="spearman")
cor(rank(vec1), rank(vec2))

cor(vec1, vec2, method="kendall")
cmpMat1 <- outer(vec1, vec1, ">")
cmpMat2 <- outer(vec2, vec2, ">")
selMat  <- upper.tri(cmpMat1)
nCP     <- sum((cmpMat1 == cmpMat2)[selMat])
nDP     <- sum((cmpMat1 != cmpMat2)[selMat])
nSubj   <- length(vec1)
nPairs  <- choose(nSubj, 2)
(tau    <- (nCP-nDP) / nPairs)

cor.test(vec1, vec2, method="spearman")
sum((rank(vec1)-rank(vec2))^2)

cor.test(vec1, vec2, method="kendall")

##### 6.3.2
vec1 <- c(100, 76, 56, 99, 50, 62, 36, 69, 55,  17)
vec2 <- c(42,  74, 22, 99, 73, 44, 10, 68, 19, -34)
cTab <- table(vec1, vec2)
N    <- sum(cTab)

library(vcd)
assocstats(cTab)

expected  <- outer(rowSums(cTab), colSums(cTab)) / N
(chisqVal <- sum((cTab-expected)^2 / expected))
chisq.test(cTab)$statistic

(phiVal  <- sqrt(chisqVal / N))

L        <- min(nrow(cTab), ncol(cTab))
phisqMax <- L-1
chisqMax <- N*phisqMax

(CrV <- sqrt(chisqVal / chisqMax))

(contCoeff <- sqrt(chisqVal / (N+chisqVal)))

##### 6.3.3
##### 6.3.3.1
categ  <- c("V", "N", "P")
rater1 <- rep(categ, c(60, 30, 10))
rater2 <- c(rep(categ, c(53, 5, 2)), rep(categ, c(11, 14, 5)),
            rep(categ, c(1, 6, 3)))
r1fac  <- factor(rater1, levels=categ)
r2fac  <- factor(rater2, levels=categ)
cTab   <- table(r1fac, r2fac)
addmargins(cTab)

library(irr)
agree(cbind(rater1, rater2))

(fObs <- sum(diag(cTab)) / sum(cTab))

rater3 <- c(rep(categ, c(48, 8, 3)), rep(categ, c(15, 10, 7)),
            rep(categ, c(3, 4, 2)))
library(irr)
agree(cbind(rater1, rater2, rater3))

r3fac <- factor(rater3, levels=categ)
cTab3 <- table(r1fac, r2fac, r3fac)
sum(diag(apply(cTab3, 3, diag))) / sum(cTab3)

##### 6.3.3.2
library(irr)
kappa2(cbind(rater1, rater2))

fObs    <- sum(diag(cTab)) / sum(cTab)
fExp    <- sum(rowSums(cTab) * colSums(cTab)) / sum(cTab)^2
(Ckappa <- (fObs-fExp) / (1-fExp))

##### 6.3.3.3
rater1 <- letters[c(4,2,2,5,2, 1,3,1,1,5, 1,1,2,1,2, 3,1,1,2,1, 5,2,2,1,1, 2,1,2,1,5)]
rater2 <- letters[c(4,2,3,5,2, 1,3,1,1,5, 4,2,2,4,2, 3,1,1,2,3, 5,4,2,1,4, 2,1,2,3,5)]
rater3 <- letters[c(4,2,3,5,2, 3,3,3,4,5, 4,4,2,4,4, 3,1,1,4,3, 5,4,4,4,4, 2,1,4,3,5)]
rater4 <- letters[c(4,5,3,5,4, 3,3,3,4,5, 4,4,3,4,4, 3,4,1,4,5, 5,4,5,4,4, 2,1,4,3,5)]
rater5 <- letters[c(4,5,3,5,4, 3,5,3,4,5, 4,4,3,4,4, 3,5,1,4,5, 5,4,5,4,4, 2,5,4,3,5)]
rater6 <- letters[c(4,5,5,5,4, 3,5,4,4,5, 4,4,3,4,5, 5,5,2,4,5, 5,4,5,4,5, 4,5,4,3,5)]

rateMat <- cbind(rater1, rater2, rater3, rater4, rater5, rater6)
library(irr)
kappam.fleiss(rateMat)

nRater   <- ncol(rateMat)
nObs     <- nrow(rateMat)
ratings  <- c(rater1, rater2, rater3, rater4, rater5, rater6)
obsFac   <- factor(rep(1:nObs, nRater))
cTab     <- xtabs(~ obsFac + ratings)
(rateTab <- prop.table(table(ratings)))
(fExp    <- sum(rateTab^2))
fObsAll  <- apply(cTab, 1, function(x) { sum(x*(x-1)) / (nRater*(nRater-1)) } )
(fObs    <- mean(fObsAll))
(Fkappa  <- (fObs-fExp) / (1-fExp))

##### 6.3.3.4
categ <- c("<10%", "11-20%", "21-30%", "31-40%", "41-50%", ">50%")
lvls  <- factor(categ, levels=categ)
tv1   <- rep(lvls, c(22, 21, 23, 16, 10, 8))
tv2   <- rep(rep(lvls, nlevels(lvls)), c(5,8,1,2,4,2, 3,5,3,5,5,0, 1,2,6,11,2,1,
                 0,1,5,4,3,3, 0,0,1,2,5,2, 0,0,1, 2,1,4))
cTab  <- table(tv1, tv2)
addmargins(cTab)
library(irr)
kappa2(cbind(tv1, tv2), weight="equal")

P          <- ncol(cTab)
(expected  <- outer(rowSums(cTab), colSums(cTab)) / sum(cTab))
(myWeights <- seq(0, 1, length.out=P))
weightsMat <- matrix(0, nrow=P, ncol=P)
for (i in 1:P) {
    weightsMat[i, ] <- c(rev(myWeights), myWeights[2:P])[(P-(i-1)):(2*P-i)]
}

weightsMat
wfObs   <-     sum(cTab * (1-weightsMat)) / sum(cTab)
wfExp   <- sum(expected * (1-weightsMat)) / sum(cTab)
(wKappa <- (wfObs-wfExp) / (1-wfExp))

##### 6.3.3.5
rater1  <- c(1, 6, 3, 2, 5, 4)
rater2  <- c(1, 5, 6, 2, 4, 3)
rater3  <- c(2, 3, 6, 5, 4, 1)
ratings <- cbind(rater1, rater2, rater3)

library(irr)
kendall(ratings)

rankMat  <- ratings
nObj     <- nrow(rankMat)
nRater   <- ncol(rankMat)
(rankSum <- apply(rankMat, 1, sum))

(S <- sum((rankSum - (nRater*(nObj+1) / 2))^2))
(W <- 12*S / (nRater^2 * nObj*(nObj^2 - 1)))

(chisqVal <- nRater * (nObj-1) * W)
(pVal     <- 1-pchisq(chisqVal, nObj-1))

##### 6.3.3.6
library(irr)
kripp.alpha(rbind(tv1, tv2), method="nominal")
kripp.alpha(rbind(tv1, tv2), method="interval")

#####---------------------------------------------------------------------------
##### 6.4 Tests auf bereinstimmung von Verteilungen
##### 6.4.1
m0     <- 30
myData <- sample(0:100, 20, replace=TRUE)
myData <- myData[myData != m0]
## fr Daten des Buches diese Datei laden:
# load("data_signTest.Rdata")
N      <- length(myData)
(obs   <- sum(myData > m0))

(pGreater  <- 1-pbinom(obs-1, N, 0.5))
(pLess     <-   pbinom(N-obs, N, 0.5))
(pTwoSided <- 2 * min(pLess, pGreater))

##### 6.4.2
IQ    <- c(99, 131, 118, 112, 128, 136, 120, 107, 134, 122)
medH0 <- 110

wilcox.test(IQ, alternative="greater", mu=medH0)

(diffIQ   <- IQ-medH0)
(idx      <- diffIQ > 0)
(rankDiff <- rank(abs(diffIQ)))
(V        <- sum(rankDiff[idx]))
(pVal     <- 1-psignrank(V-1, n=length(IQ)))

NN    <- 3
llW   <- lapply(numeric(NN), function(x) { c(0, 1) } )
(mat  <- as.matrix(expand.grid(llW)))
rMat  <- col(mat)
(vMat <- rMat * mat)
(rS   <- rowSums(vMat))

(dTab <- table(rS))
(pTab <- dTab /(2^NN))
sum(pTab)
dsignrank(0:sum(1:NN), NN)

##### 6.4.3
DV1    <- c(87, 55, 118, 106, 113)
DV2    <- c(91, 103, 102, 102, 130)
DV     <- c(DV1, DV2)
IV     <- rep(c("A", "B"), c(length(DV1), length(DV2)))
(DVtst <- IV[order(DV)])

Ns   <- table(DVtst)
runs <- rle(DVtst)
(rr  <- length(runs$lengths))
nn1  <- Ns[1]
nn2  <- Ns[2]
N    <- sum(Ns)

muR   <- 1 + ((2*nn1*nn2) / N)
varR  <- (2*nn1*nn2 * (2*nn1*nn2 - N)) / (N^2 * (N-1))
rZ    <- (rr-muR) / sqrt(varR)
(pVal <- 1-pnorm(rZ))

##### 6.4.4
DV1 <- round(rnorm(8, mean=1, sd=2), 2)
DV2 <- round(rnorm(8, mean=3, sd=2), 2)
## fr Daten des Buches diese Datei laden:
# load("data_ksTwo.Rdata")
ks.test(DV1, DV2, alternative="greater")

sortDV1  <- sort(DV1)
sortDV2  <- sort(DV2)
sortBoth <- sort(c(DV1, DV2))

both1    <- ecdf(DV1)(sortBoth)
both2    <- ecdf(DV2)(sortBoth)
diff1    <- both1-both2
diff2    <- c(0, both1[1:(length(both1)-1)]) - both2
diffBoth <- c(diff1, diff2)

(DtwoS   <- max(abs(diffBoth)))
(Dless   <- abs(min(diffBoth)))
(Dgreat  <- abs(max(diffBoth)))

ks.test(DV1, DV2, alternative="less"     )$statistic
ks.test(DV1, DV2, alternative="two.sided")$statistic

xRange <- c(min(sortBoth)-1, max(sortBoth)+1)

dev.new(width=7, height=7)
plot(ecdf(DV1), xlim=xRange, main="Kolmogorov-Smirnov-Test fr zwei Stichproben",
     xlab=NA, col.points="blue", col.hor="blue", lwd=2, cex.lab=1.4)
par(new=TRUE)
plot(ecdf(DV2), xlim=xRange, main=NA, xaxt="n", xlab=NA, ylab=NA, lwd=2,
     col.points="red", col.hor="red")
X  <- rbind(sortBoth, sortBoth)
Y1 <- rbind(both1, both2)
Y2 <- rbind(c(0, both1[1:(length(both1)-1)]), both2)
matlines(X, Y1, col="darkgray", lty=1, lwd=2)
matlines(X, Y2, col="darkgray", lty=1, lwd=2)
legend(x="bottomright", legend=c("kumulierte relative Hufigkeiten 1",
       "kumulierte relative Hufigkeiten 2"), col=c("blue", "red"), lwd=2)

##### 6.4.5
rtCntrl <- c(85, 106, 118, 81, 138, 90, 112, 119, 107, 95, 88, 103)
rtDrug  <- c(96, 105, 104, 108, 86, 84, 99, 101, 78, 124, 121, 97, 129, 87, 109)
wilcox.test(rtCntrl, rtDrug, alternative="greater")

n1      <- length(rtCntrl)
n2      <- length(rtDrug)
IV      <- factor(rep(1:2, c(n1, n2)), labels=c("control", "drug"))
rtAll   <- c(rtCntrl, rtDrug)
rankAll <- rank(rtAll)
(W      <- sum(rankAll[IV == "control"]) - sum(1:n1))
(pVal   <- 1-pwilcox(W-1, m=n1, n=n2))

(U <- sum(outer(rtCntrl, rtDrug, ">=")))

##### 6.4.6

##### 6.4.7
IQ1   <- c( 99, 131, 118, 112, 128, 136, 120, 107, 134, 122)
IQ2   <- c(134, 103, 127, 121, 139, 114, 121, 132)
IQ3   <- c(120, 133, 110, 141, 118, 124, 111, 138, 120)
IQ4   <- c(117, 125, 140, 109, 128, 137, 110, 138, 127, 141, 119, 148)
IQall <- c(IQ1, IQ2, IQ3, IQ4)
Ni    <- c(length(IQ1), length(IQ2), length(IQ3), length(IQ4))
N     <- sum(Ni)
IV    <- factor(rep(1:4, Ni), labels=c("I", "II", "III", "IV"))
KWdf  <- data.frame(id=1:N, IV, DV=IQall)
kruskal.test(DV ~ IV, data=KWdf)

rankAll   <- rank(IQall)
(rankSumI <- tapply(rankAll, IV, sum))
(H        <- (12 / (N*(N+1))) * sum(rankSumI^2/Ni) - 3*(N+1))
(pVal     <- 1-pchisq(H, nlevels(IV)-1))

##### 6.4.8
rpmCaff <- c(14, 13, 12, 11, 10)
rpmDS   <- c(11, 12, 13, 14, 15)
rpmDD   <- c(16, 15, 14, 13, 12)
rpmPlac <- c(13, 12, 11, 10,  9)
rpmAll  <- c(rpmCaff, rpmDS, rpmDD, rpmPlac)
nBl     <- length(rpmCaff)
P       <- 4
IV      <- factor(rep(1:P, each=nBl),
                  labels=c("Caffeine", "Single", "Double", "Placebo"))
blocks  <- factor(rep(1:nBl, P))
friedman.test(rpmAll, IV, blocks)

(rpmMat   <- cbind(rpmCaff, rpmDS, rpmDD, rpmPlac))
(rankMat  <- t(apply(rpmMat, 1, rank)))
(rankSumJ <- apply(rankMat, 2, sum))
(S        <- (12 / (nBl*P*(P+1))) * sum(rankSumJ^2) - 3*nBl*(P+1))
(pVal     <- 1-pchisq(S, P-1))

##### 6.4.9
pref  <- c(1,1,0,1,0, 0,1,0,0,1, 1,0,1,0,0, 1,1,1,1,1, 0,1,0,0,0,
           1,0,1,1,1, 0,0,0,0,0, 1,1,1,1,0, 0,1,0,1,1, 1,0,1,0,0)
nSubj <- 10
year  <- factor(rep(1981:1985, nSubj))
P     <- nlevels(year)
id    <- factor(rep(1:nSubj, each=P))

library(coin)
symmetry_test(pref ~ year | id, teststat="quad")

prefMat <- matrix(pref, nrow=nSubj, ncol=P, byrow=TRUE)
rSum    <- apply(prefMat, 1, sum)
cSum    <- apply(prefMat, 2, sum)
(Q      <- (P*(P-1)*sum((cSum-mean(cSum))^2)) / (P*sum(rSum) - sum(rSum^2)))
(pVal   <- 1-pchisq(Q, P-1))

##### 6.4.10
categ <- 1:3
drug  <- rep(categ, c(30, 50, 20))
plac  <- rep(rep(categ, length(categ)), c(14,7,9, 5,26,19, 1,7,12))
library(FSA)
summary(age.comp(drug, plac), what="symmetry")

cTabBow   <- table(drug, plac)
sqDiffs   <- (cTabBow-t(cTabBow))^2 / (cTabBow+t(cTabBow))
P         <- ncol(cTabBow)
(chisqVal <- sum(sqDiffs[upper.tri(cTabBow)]))
(BowDf    <- choose(P, 2))
P*(P-1)/2
(pVal     <- 1-pchisq(chisqVal, BowDf))

##### 6.4.11
nSubj     <- 20
smPre     <- rbinom(nSubj, size=1, p=0.6)
smPost    <- rbinom(nSubj, size=1, p=0.4)
smPreFac  <- factor(smPre,  levels=0:1, labels=c("no", "yes"))
smPostFac <- factor(smPost, levels=0:1, labels=c("no", "yes"))
## fr Daten des Buches diese Datei laden:
# load("data_mcnemar.Rdata")
cTab      <- table(smPreFac, smPostFac)
addmargins(cTab)
mcnemar.test(cTab, correct=FALSE)

library(coin)
symmetry_test(cTab, teststat="quad")

P         <- ncol(cTab)
(chisqVal <- (cTab[1, 2] - cTab[2, 1])^2 / (cTab[1, 2] + cTab[2, 1]))
(pVal     <- 1-pchisq(chisqVal, P*(P-1)/2))

##### 6.4.12
library(coin)
mh_test(cTabBow)

addmargins(cTabBow)
(Nij  <- ((cTabBow+t(cTabBow)) / 2)[upper.tri(cTabBow)])
(d    <- rowSums(cTabBow) - colSums(cTabBow))
num   <- sum(Nij * rev(d^2))
denom <- 2 * sum(apply(combn(Nij, 2), 2, prod))

(chisqVal <- num / denom)
(smmhDf   <- nrow(cTabBow)-1)
(pVal     <- 1-pchisq(chisqVal, smmhDf))

S         <- -(cTabBow + t(cTabBow))
diag(S)   <- rowSums(cTabBow) + colSums(cTabBow) - 2*diag(cTabBow)
keep      <- 1:(nrow(cTabBow)-1)
(chisqVal <- t(d[keep]) %*% solve(S[keep, keep]) %*% d[keep])

