#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 22.07.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Befehle, die eine Benutzerinteraktion notwendig machen oder
## nicht erwnschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausfhren zu knnen, mssen die Pakete
## car, multcomp
## samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages(c("car", "multcomp"), repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 8 Parametrische Tests fr Dispersions- und Lageparameter von Verteilungen
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 8.1 Tests auf Varianzhomogenitt
##### 8.1.1
n1  <- 110
n2  <- 90
one <- rnorm(n1, mean=100, sd=15)
two <- rnorm(n2, mean=100, sd=13)
## fr Daten des Buches diese Datei laden:
# load("data_varTest.Rdata")
var.test(one, two)

(pValF <- 2*(1-pf(var(one)/var(two), n1-1, n2-1)))
critF1 <- qf(1-(0.05/2), n1-1, n2-1)
critF2 <- qf(   0.05/2,  n1-1, n2-1)
(ciLo  <- var(one) / (critF1*var(two)))
(ciUp  <- var(one) / (critF2*var(two)))

##### 8.1.2
##### 8.1.3
library(car)
nSubj   <- 30
nGroups <- 3
myData  <- sample(0:100, nGroups*nSubj, replace=TRUE)
group   <- factor(sample(rep(1:nGroups, each=nSubj), nGroups*nSubj, replace=FALSE))
## fr Daten des Buches diese Datei laden:
# load("data_levene.Rdata")

# ab Version 2.0 des Pakets car heit die Funktion
# levene.test() nun leveneTest()
# prfe, ob Versions-Kennziffer mit 1. beginnt
res <- sessionInfo()
if(grepl("^1\\.", res$otherPkgs$car$Version)) {
  levene.test(myData, group)    # ja: verwende alten Funktionsnamen
} else {
  leveneTest(myData, group)     # nein: verwende neuen Funktionsnamen
}

model <- lm(myData ~ group)
anova(lm(abs(residuals(model)) ~ group))

groupMeds <- tapply(myData, group, median)
absDiff   <- abs(myData - groupMeds[group])
anova(lm(absDiff ~ group))

#####---------------------------------------------------------------------------
##### 8.2 t-Tests
##### 8.2.1
nSubj  <- 100
myData <- rnorm(nSubj, 5, 20)
## fr Daten des Buches diese Datei laden:
# load("data_tOne.Rdata")
muH0   <- 0
t.test(myData, alternative="two.sided", mu=muH0)

(tVal <- (mean(myData)-muH0) / (sd(myData)/sqrt(nSubj)))
(pVal <- 2*(1-pt(tVal, nSubj-1)))

tCrit <- qt(1-(0.05/2), nSubj-1)
(ciLo <- mean(myData) - tCrit*sd(myData)/sqrt(nSubj))
(ciUp <- mean(myData) + tCrit*sd(myData)/sqrt(nSubj))

##### 8.2.2
n1    <- 18
n2    <- 21
dataM <- rnorm(n1, 180, 10)
dataW <- rnorm(n2, 175, 6)
## fr Daten des Buches diese Datei laden:
# load("data_tTwoInd.Rdata")
t.test(dataM, dataW, alternative="greater", var.equal=TRUE)

estSigDiff <- sqrt((n1+n2)/(n1*n2)) * sqrt(((n1-1)*var(dataM) + (n2-1)*var(dataW)) / (n1+n2-2))
(tVal      <- (mean(dataM)-mean(dataW)) / estSigDiff )
(pVal      <- 1-pt(tVal, n1+n2-2))

tCrit <- qt(1-0.05, n1+n2-2)
(ciLo <- mean(dataM)-mean(dataW) - tCrit*estSigDiff)

t.test(dataM, dataW, alternative="greater", var.equal=FALSE)

ss1      <- var(dataM)
ss2      <- var(dataW)
num      <- (ss1/n1 + ss2/n2)^2
denom    <- ss1^2/((n1-1)*n1^2) + ss2^2/((n2-1)*n2^2)
(dfWelch <- num/denom)
(tValW   <- (mean(dataM)-mean(dataW)) / sqrt(ss1/n1 + ss2/n2))
(pVal    <- 1-pt(tValW, dfWelch))

##### 8.2.3
pre  <- rnorm(20, mean=90,  sd=15)
post <- rnorm(20, mean=100, sd=15)
## fr Daten des Buches diese Datei laden:
# load("data_tTwoDep.Rdata")
t.test(pre, post, alternative="less", paired=TRUE)

diffVar <- pre-post
t.test(diffVar, alternative="less")

#####---------------------------------------------------------------------------
##### 8.3 Einfaktorielle Varianzanalyse (CR-p)
##### 8.3.1
contr.treatment(4)
options("contrasts")
contr.sum(4)

##### 8.3.2
yield   <- c(2349, 2554, 2278, 2476, 2242, 2124, 2649, 2106, 2107, 2185, 2095, 2070)
month   <- factor(rep(1:3, each=4), labels=c("Jan", "Feb", "Mar"))
dfCrop  <- data.frame(month, yield)
oneway.test(yield ~ month, data=dfCrop, var.equal=TRUE)

P       <- nlevels(month)
N       <- length(yield)
groupNs <- tapply(yield, month, length)
groupVs <- tapply(yield, month, var)
groupMs <- tapply(yield, month, mean)
grandM  <- sum((groupNs/N) * groupMs)
SSw     <- sum((groupNs-1) * groupVs)
SSb     <- sum(groupNs * (groupMs-grandM)^2)
MSw     <- SSw / (N-P)
MSb     <- SSb / (P-1)
(Fval   <- MSb / MSw)
(pVal   <- 1-pf(Fval, P-1, N-P))

oneway.test(yield ~ month, data=dfCrop, var.equal=FALSE)

##### 8.3.3
## fr Daten des Buches diese Datei laden:
# load("data_crpYield.Rdata")
(aovCRp <- aov(yield ~ month, dfCrop))
summary(aovCRp)
model.tables(aovCRp, type="means")
dev.new(width=5, height=5)
plot.design(yield ~ month, fun=mean, data=dfCrop,
            main="Mittelwerte getrennt nach Gruppen", cex.lab=1.2)

##### 8.3.4
## fr Daten des Buches diese Datei laden:
# load("data_crpYield.Rdata")
dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
plot(fitted(aovCRp), residuals(aovCRp), pch=20, xlab="Vorhersage",
     ylab="Residuen", main="Residuen gegen Vorhersage", cex=2)
abline(h=0, col="blue", lwd=2)
plot(residuals(aovCRp) ~ aovCRp$model$month, main="Fehler vs. Stufen")
qqnorm(residuals(aovCRp), pch=20, cex=2)
qqline(residuals(aovCRp), col="red", lwd=2)
par(op)

##### 8.3.5
## fr Daten des Buches diese Datei laden:
# load("data_crpYield.Rdata")
anova(lm(yield ~ month, data=dfCrop))

nSubj   <- 100
pred1   <- rnorm(nSubj, 175, 7)
pred2   <- rnorm(nSubj, 30, 8)
crit    <- 0.5*pred1 - 0.3*pred2 + 10 + rnorm(nSubj, 0, 6)
## fr Daten des Buches diese Datei laden:
# load("data_anovaComp.Rdata")
(sumRes <- summary(lm(crit ~ pred1 + pred2)))

(anovaRes <- anova(lm(crit ~ pred1), lm(crit ~ pred1 + pred2)))

dfMod1 <- nSubj - (1+1)
dfMod2 <- nSubj - (2+1)
MSMod2 <- anovaRes[["Sum of Sq"]][2] / (dfMod1 - dfMod2)
MSE    <- sum(residuals(sumRes)^2 / dfMod2)
MSMod2 / MSE
(sumRes$coefficients[3, 3])^2

##### 8.3.6
##### 8.3.6.1
## fr Daten des Buches diese Datei laden:
# load("data_crpYield.Rdata")
library(multcomp)
aovCRp  <- aov(yield ~ month, data=dfCrop)
cntrMat <- rbind("0.5*(Jan+Feb)-Mar"=c(1/2, 1/2, -1))
summary(glht(aovCRp, linfct=mcp(month=cntrMat), alternative="greater"))

anovaCRp <- anova(lm(yield ~ month, dfCrop))
MSw      <- anovaCRp[["Mean Sq"]][2]
P        <- nlevels(dfCrop$month)
groupMs  <- tapply(dfCrop$yield, dfCrop$month, mean)
groupNs  <- tapply(dfCrop$yield, dfCrop$month, length)
dfSSw    <- sum(groupNs)-P
psiHat   <- sum(cntrMat[1, ] * groupMs)
lenSq    <- sum((cntrMat[1, ]^2) * (1/groupNs))
statTc   <- psiHat / sqrt(lenSq*MSw)
abs(statTc)
(critT   <- qt(1-0.05, dfSSw))
(pValT   <- 1-pt(abs(statTc), dfSSw))
(ciLo    <- psiHat - critT*sqrt(lenSq*MSw))

cntrMat  <- rbind("Jan-0.5*(Feb+Mar)"=c(  1, -1/2, -1/2),
        "Feb-Jan"          =c( -1,    1,    0),
        "0.5*(Jan+Feb)-Mar"=c(1/2,  1/2,   -1),
        "Feb-Mar"          =c(  0,    1,   -1))
summary(glht(aovCRp, linfct=mcp(month=cntrMat), alternative="greater"),
        test=adjusted("none"))

(psiHats <- cntrMat %*% groupMs)
alphaAdj <- 0.05/nrow(cntrMat)
lenSqs   <- cntrMat^2 %*% (1/groupNs)
statTcs  <- psiHats / sqrt(lenSqs*MSw)
critTs   <- rep(qt(1-alphaAdj, dfSSw), nrow(cntrMat))
pValTs   <- 1-pt(abs(statTcs), dfSSw)
(resDf   <- data.frame(statTcs, critTs, pVals=pValTs,
                       alphaAdj=rep(alphaAdj, nrow(cntrMat)),
                       sig=abs(statTcs)>critTs))

##### 8.3.6.2
## fr Daten des Buches diese Datei laden:
# load("data_crpYield.Rdata")
dfSSb  <- P-1
(statF <- psiHat^2 / (lenSq*MSw))
(critF <- dfSSb*qf(1-0.05, df1=dfSSb, df2=dfSSw))
(pValF <- 1-pf(statF/dfSSb, dfSSb, dfSSw))

##### 8.3.6.3
nSubj <- 50
DVa   <- rnorm(nSubj, 0,   1)
DVb   <- rnorm(nSubj, 0.3, 1)
DVc   <- rnorm(nSubj, 0.6, 1)
DVd   <- rnorm(nSubj, 1.0, 1)
DV    <- c(DVa, DVb, DVc, DVd)
IV    <- factor(rep(LETTERS[1:4], each=nSubj))
## fr Daten des Buches diese Datei laden:
# load("data_crpTukey.Rdata")
anova(lm(DV ~ IV))

pairwise.t.test(DV, IV, p.adjust.method="bonferroni")
pairwise.t.test(DV, IV, p.adjust.method="none", pool.sd=FALSE)

t.test(DV[IV %in% c("A", "B")] ~ IV[IV %in% c("A", "B")])
t.test(DV[IV %in% c("A", "C")] ~ IV[IV %in% c("A", "C")])
t.test(DV[IV %in% c("A", "D")] ~ IV[IV %in% c("A", "D")])
t.test(DV[IV %in% c("B", "C")] ~ IV[IV %in% c("B", "C")])
t.test(DV[IV %in% c("B", "D")] ~ IV[IV %in% c("B", "D")])
t.test(DV[IV %in% c("C", "D")] ~ IV[IV %in% c("C", "D")])

##### 8.3.6.4
## fr Daten des Buches diese Datei laden:
# load("data_crpTukey.Rdata")
(tHSD <- TukeyHSD(aov(DV ~ IV)))

Ms  <- tapply(DV, IV, mean)
P   <- nlevels(IV)
dfE <- (nSubj*P)-P

anovaRes <- anova(lm(DV ~ IV))
MSE <- anovaRes[["Mean Sq"]][2]

diffMat <- outer(Ms, Ms, "-")
(diffs  <- diffMat[lower.tri(diffMat)])
qTs     <- abs(diffs) / sqrt(MSE / nSubj)
(pVals  <- 1-ptukey(qTs, P, dfE))

tWidth  <- qtukey(1-0.05, P, dfE) * sqrt(MSE / nSubj)
diffs - tWidth
diffs + tWidth

dev.new(width=5, height=5)
op <- par(cex.lab=1.2)
plot(tHSD)
par(op)

#####---------------------------------------------------------------------------
##### 8.4 Einfaktorielle Varianzanalyse mit abhngigen Gruppen (RB-p)
##### 8.4.1
nSubj   <- 10
P       <- 3
id      <- factor(rep(1:nSubj, P))
timeFac <- factor(rep(1:P, each=nSubj))
DV_t1   <- round(rnorm(nSubj, -0.3, 1), 2)
DV_t2   <- round(rnorm(nSubj, -0.2, 1), 2)
DV_t3   <- round(rnorm(nSubj,  0.5, 1), 2)
dfRBpL  <- data.frame(id, timeFac, DV=c(DV_t1, DV_t2, DV_t3))
## fr Daten des Buches diese Datei laden:
# load("data_rbp.Rdata")
aovRBp  <- aov(DV ~ timeFac + Error(id/timeFac), data=dfRBpL)
summary(aovRBp)

anova(lm(DV ~ id + timeFac + id:timeFac, data=dfRBpL))

DVctr  <- dfRBpL$DV - ave(dfRBpL$DV, dfRBpL$id, FUN=mean)
tfMs   <- tapply(DVctr, dfRBpL$timeFac, mean)
grandM <- mean(tfMs)
SSb    <- sum(nSubj * (tfMs-grandM)^2)

IDxTF  <- DVctr - ave(DVctr, dfRBpL$timeFac, FUN=mean) + grandM
SSE    <- sum(IDxTF^2)
dfSSb  <- P-1
dfSSE  <- (nSubj-1) * (P-1)

(MSb   <- SSb / dfSSb)
(MSE   <- SSE / dfSSE)
(Fval  <- MSb / MSE)
(pVal  <- 1-pf(Fval, P-1, (nSubj-1)*(P-1)))

##### 8.4.2
## fr Daten des Buches diese Datei laden:
# load("data_rbp.Rdata")
dataMat <- cbind(DV_t1, DV_t2, DV_t3)
nSubj   <- nrow(dataMat)
S       <- var(dataMat)
P       <- nrow(S)
mdS     <- mean(diag(S))
mS      <- mean(S)
mSr     <- rowMeans(S)
N       <- P^2 * (mdS-mS)^2
D       <- (P-1)*(sum(S^2) - 2*P*sum(mSr^2) + P^2*mS^2)

(epsGG  <- N/D)
(epsHF  <- (nSubj*(P-1)*epsGG - 2) / ((P-1) * ((nSubj-1) - ((P-1)*epsGG))))

epsHF   <- ifelse(epsHF > 1, 1, epsHF)

dfEpsGG <- c(dfSSb, dfSSE) * epsGG
(pEpsGG <- 1-pf(Fval, df1=dfEpsGG[1], df2=dfEpsGG[2]))

dfEpsHF <- c(dfSSb, dfSSE) * epsHF
(pEpsHF <- 1-pf(Fval, df1=dfEpsHF[1], df2=dfEpsHF[2]))

dfRBpL$IDxTF <- IDxTF
errMat       <- as.matrix(unstack(dfRBpL, IDxTF ~ timeFac))
Serr         <- cov(errMat)
(epsGG       <- (1 / (P-1)) * sum(diag(Serr))^2 / sum(Serr^2))

##### 8.4.3
## fr Daten des Buches diese Datei laden:
# load("data_rbp.Rdata")
dfRBpW   <- data.frame(DV_t1, DV_t2, DV_t3)
modelRBp <- lm(cbind(DV_t1, DV_t2, DV_t3) ~ 1, data=dfRBpW)
intraRBp <- data.frame(timeFac=factor(1:3))

library(car)
(AnovaRBp <- Anova(modelRBp, idata=intraRBp, idesign=~timeFac))
summary(AnovaRBp, multivariate=FALSE, univariate=TRUE)

anova(modelRBp, idata=intraRBp, X=~1, test="Spherical")

#####---------------------------------------------------------------------------
##### 8.5 Zweifaktorielle Varianzanalyse (CRF-pq)
##### 8.5.1
nSubj  <- 8
P      <- 2
Q      <- 3
IV1    <- factor(rep(1:P, nSubj*Q))
IV2    <- factor(rep(1:Q, each=nSubj*P))
IVcomb <- factor(c(rep(c("1-1", "2-1"), nSubj), rep(c("1-2", "2-2"), nSubj),
                   rep(c("1-3", "2-3"), nSubj)))
DV     <- rnorm(nSubj*P*Q, 0, 1)
DV[IV1 == 1] <- DV[IV1 == 1] + 0.5
DV[IV1 == 2] <- DV[IV1 == 2] - 0.5
DV[IV2 == 1] <- DV[IV2 == 1] - 0.5
DV[IV2 == 3] <- DV[IV2 == 3] + 0.5
dfCRFpq <- data.frame(id=1:(P*Q*nSubj), IV1, IV2, IVcomb, DV)

## fr Daten des Buches diese Datei laden:
# load("data_crfpq.Rdata")
summary(aov(DV ~ IV1*IV2, data=dfCRFpq))

dev.new(width=7, height=4)
op <- par(mfrow=c(1, 2), cex.main=0.9)
plot.design(DV ~ IV1*IV2, data=dfCRFpq, main="Gruppen-Mittelwerte")
interaction.plot(IV1, IV2, DV, main="Mittelwertsverlufe",
                 col=c("red", "blue", "green"), lwd=2)
par(op)

##### 8.5.2
mdP   <- 3
mdQ   <- 3
g11   <- c(41, 43, 50)
g12   <- c(51, 43, 53, 54, 46)
g13   <- c(45, 55, 56, 60, 58, 62, 62)
g21   <- c(56, 47, 45, 46, 49)
g22   <- c(58, 54, 49, 61, 52, 62)
g23   <- c(59, 55, 68, 63)
g31   <- c(43, 56, 48, 46, 47)
g32   <- c(59, 46, 58, 54)
g33   <- c(55, 69, 63, 56, 62, 67)
mdDV  <- c(g11, g12, g13, g21, g22, g23, g31, g32, g33)
mdIV1 <- factor(rep(1:mdP, c(3+5+7, 5+6+4, 5+4+6)))
mdIV2 <- factor(rep(rep(1:mdQ, mdP), c(3,5,7, 5,6,4, 5,4,6)))
dfMD  <- data.frame(IV1=mdIV1, IV2=mdIV2, DV=mdDV)

anova(lm(DV ~ IV1 + IV2 + IV1:IV2, data=dfMD))

SSI1 <- anova(lm(DV ~ 1,         data=dfMD),
              lm(DV ~ IV1,                 data=dfMD))
SSI2 <- anova(lm(DV ~ IV1,       data=dfMD),
              lm(DV ~ IV1 + IV2,           data=dfMD))
SSIi <- anova(lm(DV ~ IV1 + IV2, data=dfMD),
              lm(DV ~ IV1 + IV2 + IV1:IV2, data=dfMD))

SSI1[["Sum of Sq"]][2]
SSI2[["Sum of Sq"]][2]
SSIi[["Sum of Sq"]][2]

SSIt <- anova(lm(DV ~ 1,         data=dfMD),
              lm(DV ~ IV1 + IV2 + IV1:IV2, data=dfMD))
SSIt[["Sum of Sq"]][2]
SSI1[["Sum of Sq"]][2] + SSI2[["Sum of Sq"]][2] + SSIi[["Sum of Sq"]][2]

modelIII <- lm(DV ~ IV1 + IV2 + IV1:IV2, data=dfMD,
               contrasts=list(IV1=contr.sum, IV2=contr.sum))
drop1(modelIII, ~ ., test="F")

library(car)
Anova(modelIII, type="III")

(cellMs <- tapply(dfMD$DV, list(dfMD$IV1, dfMD$IV2), mean))
(cellNs <- tapply(dfMD$DV, list(dfMD$IV1, dfMD$IV2), length))
rowMs   <- rowMeans(cellMs)
colMs   <- colMeans(cellMs)
effNj   <- mdQ / apply(1/cellNs, 1, sum)
effNk   <- mdP / apply(1/cellNs, 2, sum)
gM1     <- sum(effNj*rowMs) / sum(effNj)
gM2     <- sum(effNk*colMs) / sum(effNk)
(SSIII1 <- mdP * sum(effNj * (rowMs-gM1)^2))
(SSIII2 <- mdQ * sum(effNk * (colMs-gM2)^2))
(SSE    <- sum((dfMD$DV - ave(dfMD$DV, dfMD$IV1, dfMD$IV2, FUN=mean))^2))

dfSS1  <- mdP-1
dfSS2  <- mdQ-1
dfSSE  <- sum(cellNs-1)
(Fval1 <- (SSIII1/dfSS1) / (SSE/dfSSE))
(Fval2 <- (SSIII2/dfSS2) / (SSE/dfSSE))
(pVal1 <- 1-pf(Fval1, dfSS1, dfSSE))
(pVal2 <- 1-pf(Fval2, dfSS2, dfSSE))

##### 8.5.3
## fr Daten des Buches diese Datei laden:
# load("data_crfpq.Rdata")
CRFpq1 <- anova(lm(DV ~ IVcomb,  data=dfCRFpq))
CRFpq2 <- anova(lm(DV ~ IV1*IV2, data=dfCRFpq))
isTRUE(all.equal(CRFpq1[["Mean Sq"]][2], CRFpq2[["Mean Sq"]][4]))

library(multcomp)
aovCRFpq <- aov(DV ~ IVcomb, data=dfCRFpq)
cntrMat  <- rbind("contrast 01"=c(1/2, 1/2, -1/4, -1/4, -1/4, -1/4))
summary(glht(aovCRFpq, linfct=mcp(IVcomb=cntrMat), alternative="greater"))

cellMs <- tapply(dfCRFpq$DV, dfCRFpq$IVcomb, mean)
MSE    <- CRFpq2[["Mean Sq"]][4]
dfSSE  <- (nSubj-1)*P*Q
psiHat <- sum(cntrMat[1, ] * cellMs)
lenSq  <- (1/nSubj) * sum(cntrMat[1, ]^2)
statTc <- psiHat / sqrt(lenSq*MSE)
abs(statTc)
(critT <- qt(1-0.05, dfSSE))
(pValT <- 1-pt(abs(statTc), dfSSE))
(ciLo  <- psiHat - critT*sqrt(lenSq*MSE))

cntrMat <- rbind("contrast 01"=c(1/2, 1/2, -1/4, -1/4, -1/4, -1/4),
        "contrast 02"=c(0,     1,   -1,    0,    0,    0),
        "contrast 03"=c(0,     0,    0,  1/2,   -1,  1/2))
summary(glht(aovCRFpq, linfct=mcp(IVcomb=cntrMat), alternative="greater"),
        test=adjusted("none"))

psiHats  <- cntrMat %*% cellMs
lenSqs   <- (1/nSubj) * cntrMat^2 %*% (rep(1, ncol(cntrMat)))
alphaAdj <- 0.05/nrow(cntrMat)
statTs   <- psiHats / sqrt(lenSqs*MSE)
critTs   <- rep(qt(1-alphaAdj, dfSSE), nrow(cntrMat))
pValTs   <- 1-pt(abs(statTs), dfSSE)
(resDf   <- data.frame(statTs, critTs, pVals=pValTs,
                       alphaAdj=rep(alphaAdj, nrow(cntrMat)),
                       sig=abs(statTs)>critTs))

meansA   <- tapply(dfCRFpq$DV, dfCRFpq$IV1, mean)
cntrVecA <- c(1, -1)
psiHatA  <- sum(cntrVecA * meansA)
lenSqA   <- (1/(Q*nSubj)) * sum(cntrVecA^2)
statA    <- psiHatA / sqrt(lenSqA*MSE)

meansB   <- tapply(dfCRFpq$DV, dfCRFpq$IV2, mean)
cntrVecB <- c(1, -1/2, -1/2)
psiHatB  <- sum(cntrVecB * meansB)
lenSqB   <- (1/(P*nSubj)) * sum(cntrVecB^2)
statB    <- psiHatB / sqrt(lenSqB*MSE)

##### 8.5.4
## fr Daten des Buches diese Datei laden:
# load("data_crfpq.Rdata")
(statF <- psiHat^2 / (lenSq*MSE))
dfSSba <- P*Q - 1
(critF <- dfSSba*qf(1-0.05, df1=dfSSba, df2=dfSSE))
(pValF <- 1-pf(statF/dfSSba, dfSSba, dfSSE))

#####---------------------------------------------------------------------------
##### 8.6 Zweifaktorielle Varianzanalyse mit zwei intra-Gruppen Faktoren (RBF-pq)
##### 8.6.1
nSubj    <- 10
P        <- 2
Q        <- 3
id       <- factor(rep(1:nSubj, P*Q))
timeFac1 <- factor(rep(rep(1:P, each=nSubj), Q))
timeFac2 <- factor(rep(rep(1:Q, each=nSubj*P)))
DV_t11   <- round(rnorm(nSubj, -0.8, 1), 2)
DV_t12   <- round(rnorm(nSubj, -0.7, 1), 2)
DV_t13   <- round(rnorm(nSubj,  0.0, 1), 2)
DV_t21   <- round(rnorm(nSubj,  0.2, 1), 2)
DV_t22   <- round(rnorm(nSubj,  0.3, 1), 2)
DV_t23   <- round(rnorm(nSubj,  1.0, 1), 2)
dfRBFpqL <- data.frame(id, timeFac1, timeFac2,
                       DV=c(DV_t11, DV_t21, DV_t12, DV_t22, DV_t13, DV_t23))

## fr Daten des Buches diese Datei laden:
# load("data_rbfpq.Rdata")
aovRBFpq <- aov(DV ~ timeFac1*timeFac2 + Error(id/(timeFac1*timeFac2)),
                data=dfRBFpqL)
summary(aovRBFpq)

dfG  <- dfRBFpqL
mDf1 <- aggregate(dfG$DV, list(dfG$id, dfG$timeFac1), mean)
mDf2 <- aggregate(dfG$DV, list(dfG$id, dfG$timeFac2), mean)
dDfI <- aggregate(dfG$DV, list(dfG$id, dfG$timeFac2), diff)

summary(aov(x ~ Group.2 + Error(Group.1/Group.2), data=mDf1))
summary(aov(x ~ Group.2 + Error(Group.1/Group.2), data=mDf2))
summary(aov(x ~ Group.2 + Error(Group.1/Group.2), data=dDfI))

cellMs  <- tapply(dfG$DV, list(dfG$timeFac1, dfG$timeFac2), mean)
tf1Ms   <- tapply(dfG$DV, dfG$timeFac1, mean)
tf2Ms   <- tapply(dfG$DV, dfG$timeFac2, mean)
grandM  <- mean(cellMs)

TF1xTF2 <- c(sweep(sweep(cellMs, 1, tf1Ms, "-"), 2, tf2Ms, "-")) + grandM
(SSI    <- nSubj * sum(TF1xTF2^2))

cellMsL  <- ave(dfG$DV,         dfG$timeFac1, dfG$timeFac2, FUN=mean)
blTf1MsL <- ave(dfG$DV, dfG$id, dfG$timeFac1,               FUN=mean)
blTf2MsL <- ave(dfG$DV, dfG$id,               dfG$timeFac2, FUN=mean)
blockMsL <- ave(dfG$DV, dfG$id,                             FUN=mean)
tf1MsL   <- ave(dfG$DV,         dfG$timeFac1,               FUN=mean)
tf2MsL   <- ave(dfG$DV,                       dfG$timeFac2, FUN=mean)

IDxTF1xTF2 <- dfG$DV - blTf1MsL - blTf2MsL - cellMsL + blockMsL + tf1MsL + tf2MsL - grandM
(SSE       <- sum(IDxTF1xTF2^2))

dfSSI   <- (P-1) * (Q-1)
dfSSE   <- (P-1) * (Q-1) * (nSubj-1)

(FvalI  <- (SSI/dfSSI) / (SSE/dfSSE))
(pValI  <- 1-pf(FvalI, dfSSI, dfSSE))

##### 8.6.2
## fr Daten des Buches diese Datei laden:
# load("data_rbfpq.Rdata")
dataMat <- tapply(dfG$DV, list(dfG$id, dfG$timeFac2), mean)
nSubj   <- nrow(dataMat)
S       <- var(dataMat)
Q       <- nrow(S)
mdS     <- mean(diag(S))
mS      <- mean(S)
mSr     <- rowMeans(S)
N       <- Q^2 * (mdS-mS)^2
D       <- (Q-1)*(sum(S^2) - 2*Q*sum(mSr^2) + Q^2*mS^2)

(epsGG2 <- N/D)
(epsHF2 <- (nSubj*(Q-1)*epsGG2 - 2) / ((Q-1) * ((nSubj-1) - ((Q-1)*epsGG2))))

dfG     <- cbind(dfG, IDxTF1xTF2)
errMat  <- as.matrix(unstack(dfG, IDxTF1xTF2 ~ timeFac2))
Serr    <- cov(errMat)

(epsGGI <- (1 / (Q-1)) * sum(diag(Serr))^2 / sum(Serr^2))
(epsHFI <- (nSubj*(Q-1)*epsGGI - 2) / ((Q-1) * ((nSubj-1) - ((Q-1)*epsGGI))))

##### 8.6.3
## fr Daten des Buches diese Datei laden:
# load("data_rbfpq.Rdata")
dfRBFpqW    <- data.frame(DV_t11, DV_t21, DV_t12, DV_t22, DV_t13, DV_t23)
modelRBFpq  <- lm(cbind(DV_t11, DV_t21, DV_t12, DV_t22, DV_t13, DV_t23) ~ 1,
                  data=dfRBFpqW)
(intraRBFpq <- data.frame(timeFac1=factor(rep(1:P, Q)),
                          timeFac2=factor(rep(1:Q, each=P))))

library(car)
AnovaRBFpq  <- Anova(modelRBFpq, idata=intraRBFpq, idesign=~timeFac1*timeFac2)
summary(AnovaRBFpq, multivariate=FALSE, univariate=TRUE)

anova(modelRBFpq, M=~timeFac1 + timeFac2, X=~timeFac2,
                  idata=intraRBFpq, test="Spherical")
anova(modelRBFpq, M=~timeFac1 + timeFac2, X=~timeFac1,
                  idata=intraRBFpq, test="Spherical")
anova(modelRBFpq,                         X=~timeFac1 + timeFac2,
                  idata=intraRBFpq, test="Spherical")

##### 8.6.4

#####---------------------------------------------------------------------------
##### 8.7 Zweifaktorielle Varianzanalyse mit Split-Plot-Design (SPF-pq)
##### 8.7.1
nSubj    <- 10
P        <- 3
Q        <- 3
id       <- factor(rep(1:(P*nSubj), Q))
group    <- factor(rep(LETTERS[1:P], Q*nSubj))
timeFac  <- factor(rep(1:Q, each=P*nSubj))
DV_t1    <- round(rnorm(P*nSubj, -0.5, 1), 2)
DV_t2    <- round(rnorm(P*nSubj,  0,   1), 2)
DV_t3    <- round(rnorm(P*nSubj,  0.5, 1), 2)
dfSPFpqL <- data.frame(id, group, timeFac, DV=c(DV_t1, DV_t2, DV_t3))

## fr Daten des Buches diese Datei laden:
# load("data_spfpq.Rdata")
aovSPFpq <- aov(DV ~ group*timeFac + Error(id/timeFac), data=dfSPFpqL)
summary(aovSPFpq)

##### 8.7.2
## fr Daten des Buches diese Datei laden:
# load("data_spfpq.Rdata")
mDf <- aggregate(dfSPFpqL$DV, list(dfSPFpqL$id, dfSPFpqL$group), FUN=mean)
summary(aov(x ~ Group.2, data=mDf))

cellMs  <- ave(dfSPFpqL$DV,              dfSPFpqL$group, dfSPFpqL$timeFac, FUN=mean)
blockMs <- ave(dfSPFpqL$DV, dfSPFpqL$id,                                   FUN=mean)
groupMs <- ave(dfSPFpqL$DV,              dfSPFpqL$group,                   FUN=mean)

IDxTF    <- dfSPFpqL$DV - blockMs - cellMs + groupMs
sum(IDxTF^2)
dfSPFpqL <- cbind(dfSPFpqL, IDxTF)

errMat   <- as.matrix(unstack(dfSPFpqL, IDxTF ~ timeFac))
Serr     <- cov(errMat)
(epsGG   <- (1 / (Q-1)) * sum(diag(Serr))^2 / sum(Serr^2))

nSubjEps <- nrow(errMat) - (P-1)
(epsHF   <- (nSubjEps*(Q-1)*epsGG - 2) / ((Q-1) * ((nSubjEps-1) - ((Q-1)*epsGG))))

##### 8.7.3
## fr Daten des Buches diese Datei laden:
# load("data_spfpq.Rdata")
groupW     <- factor(rep(LETTERS[1:P], nSubj))
dfSPFpqW   <- data.frame(groupW, DV_t1, DV_t2, DV_t3)

modelSPFpq <- lm(cbind(DV_t1, DV_t2, DV_t3) ~ groupW, data=dfSPFpqW)
intraSPFpq <- data.frame(timeFac=factor(1:Q))

library(car)
AnovaSPFpq <- Anova(modelSPFpq, idata=intraSPFpq, idesign=~timeFac)
summary(AnovaSPFpq, multivariate=FALSE, univariate=TRUE)

anova(modelSPFpq, X=~1, idata=intraSPFpq, test="Spherical")
anova(modelSPFpq, M=~1, idata=intraSPFpq, test="Spherical")

##### 8.7.4
##### 8.7.5
nSubj      <- 10
P          <- 2
Q          <- 3
R          <- 2
id         <- factor(rep(1:(P*nSubj), Q*R))
group      <- factor(rep(LETTERS[1:P], Q*R*nSubj))
timeFac1   <- factor(rep(1:Q, each=P*R*nSubj))
timeFac2   <- factor(rep(rep(1:R, each=P*nSubj), Q))
DV_t11     <- round(rnorm(P*nSubj, 8,  2), 2)
DV_t12     <- round(rnorm(P*nSubj, 10, 2), 2)
DV_t21     <- round(rnorm(P*nSubj, 13, 2), 2)
DV_t22     <- round(rnorm(P*nSubj, 15, 2), 2)
DV_t31     <- round(rnorm(P*nSubj, 13, 2), 2)
DV_t32     <- round(rnorm(P*nSubj, 15, 2), 2)
dfSPFp.qrL <- data.frame(id, group, timeFac1, timeFac2,
                         DV=c(DV_t11, DV_t12, DV_t21, DV_t22, DV_t31, DV_t32))

aovSPFp.qr <- aov(DV ~ group*timeFac1*timeFac2 + Error(id/(timeFac1*timeFac2)),
                  data=dfSPFp.qrL)
summary(aovSPFp.qr)

groupW        <- factor(rep(LETTERS[1:P], nSubj))
dfSPFp.qrW    <- data.frame(groupW, DV_t11, DV_t12, DV_t21, DV_t22, DV_t31, DV_t32)

modelSPFp.qr  <- lm(cbind(DV_t11, DV_t12, DV_t21, DV_t22, DV_t31, DV_t32) ~ groupW,
                    data=dfSPFp.qrW)
(intraSPFp.qr <- data.frame(timeFac1=factor(rep(1:Q, each=R)),
                            timeFac2=factor(rep(1:R, Q))))

library(car)
AnovaSPFp.qr  <- Anova(modelSPFp.qr, idata=intraSPFp.qr,
                       idesign=~timeFac1*timeFac2)
summary(AnovaSPFp.qr, multivariate=FALSE, univariate=TRUE)

##### 8.7.6
nSubj   <- 10
P       <- 2
Q       <- 2
R       <- 3
id      <- factor(rep(1:(P*Q*nSubj), R))
IV1     <- factor(rep(1:P, Q*R*nSubj))
IV2     <- factor(rep(rep(1:Q, each=P*nSubj), R))
timeFac <- factor(rep(1:R, each=P*Q*nSubj))
DV_t1   <- round(rnorm(P*Q*nSubj, -3, 2), 2)
DV_t2   <- round(rnorm(P*Q*nSubj,  1, 2), 2)
DV_t3   <- round(rnorm(P*Q*nSubj,  2, 2), 2)
dfSPFpq.rL <- data.frame(id, IV1, IV2, timeFac, DV=c(DV_t1, DV_t2, DV_t3))

aovSPFpq.r <- aov(DV ~ IV1*IV2*timeFac + Error(id/timeFac), data=dfSPFpq.rL)
summary(aovSPFpq.r)

IV1W         <- factor(rep(LETTERS[1:P], Q*nSubj))
IV2W         <- factor(rep(c("+", "-"), each=P*nSubj))
dfSPFpq.rW   <- data.frame(IV1W, IV2W, DV_t1, DV_t2, DV_t3)
modelSPFpq.r <- lm(cbind(DV_t1, DV_t2, DV_t3) ~ IV1W*IV2W, data=dfSPFpq.rW)
intraSPFpq.r <- data.frame(timeFac=factor(1:R))

library(car)
AnovaSPFpq.r <- Anova(modelSPFpq.r, idata=intraSPFpq.r, idesign=~timeFac)
summary(AnovaSPFpq.r, multivariate=FALSE, univariate=TRUE)

#####---------------------------------------------------------------------------
##### 8.8 Kovarianzanalyse
##### 8.8.1
SSRIpre  <- c(18, 16, 16, 15, 14, 20, 14, 21, 25, 11)
SSRIpost <- c(12,  0, 10,  9,  0, 11,  2,  4, 15, 10)
PlacPre  <- c(18, 16, 15, 14, 20, 25, 11, 25, 11, 22)
PlacPost <- c(11,  4, 19, 15,  3, 14, 10, 16, 10, 20)
WLpre    <- c(15, 19, 10, 29, 24, 15,  9, 18, 22, 13)
WLpost   <- c(17, 25, 10, 22, 23, 10,  2, 10, 14,  7)

nSubj    <- length(SSRIpre)
IV       <- factor(rep(1:3, each=nSubj), labels=c("SSRI", "Placebo", "WL"))
P        <- nlevels(IV)
DVpre    <- c(SSRIpre,  PlacPre,  WLpre)
DVpost   <- c(SSRIpost, PlacPost, WLpost)
dfAncova <- data.frame(id=1:(P*nSubj), IV, DVpre, DVpost)

dev.new(width=5, height=5)
plot(DVpost ~ IV, main="Boxplots der Scores fr jede Gruppe", cex.lab=1.2)

modelFull <- lm(DVpost ~ IV + DVpre, data=dfAncova)
modelGrp  <- lm(DVpost ~ IV,         data=dfAncova)
modelRegr <- lm(DVpost ~      DVpre, data=dfAncova)

anova(modelGrp)
anova(modelFull)

library(car)
Anova(modelFull, type="III")
anova(modelRegr, modelFull)
anova(modelGrp,  modelFull)

X    <- DVpre
Y    <- DVpost
XMs  <- tapply(X, IV, mean)
YMs  <- tapply(Y, IV, mean)
N    <- length(Y)
Xctr <- X - ave(X, IV, FUN=mean)
Yctr <- Y - ave(Y, IV, FUN=mean)

bFull     <- cov(Xctr, Yctr) / var(Xctr)
aFull     <- YMs - bFull*XMs
YhatFull  <- bFull*X + aFull[IV]
SSEfull   <- sum((Y-YhatFull)^2)
dfSSEfull <- N-P-1
(MSEfull  <- SSEfull / dfSSEfull)

bRegr    <- cov(X, Y) / var(X)
aRegr    <- mean(Y) - bRegr*mean(X)
YhatRegr <- bRegr*X + aRegr
SSEregr  <- sum((Y-YhatRegr)^2)
(MSEregr <- SSEregr / (N-2))

grpNs    <- tapply(Y, IV, length)
grpVs    <- tapply(Y, IV, var)
grandM   <- sum((grpNs/N) * YMs)
SSEgrp   <- sum((grpNs-1) * grpVs)
(MSEgrp  <- SSEgrp / (N-P))

SSregr   <- SSEgrp - SSEfull
(MSregr  <- SSregr / 1)
(Fregr   <- MSregr / MSEfull)

SSgrp    <- SSEregr - SSEfull
(MSgrp   <- SSgrp / (P-1))
(Fgrp    <- MSgrp / MSEfull)

(sumRes   <- summary(modelFull))
iCeptSSRI <- sumRes$coefficients[1, 1]
iCeptPlac <- sumRes$coefficients[2, 1] + iCeptSSRI
iCeptWL   <- sumRes$coefficients[3, 1] + iCeptSSRI
slopeAll  <- sumRes$coefficients[4, 1]

xLims     <- c(0, max(dfAncova$DVpre))
yLims     <- c(min(iCeptSSRI, iCeptPlac, iCeptWL), max(dfAncova$DVpost))

dev.new(width=7, height=7)
plot(DVpost ~ DVpre, data=dfAncova, xlim=xLims, ylim=yLims,
     pch=rep(c(3, 17, 19), each=nSubj), col=rep(c("red", "green", "blue"), each=nSubj),
     main="Rohdaten und Regressionsgerade pro Gruppe", cex.lab=1.4)
legend(x="topleft", legend=levels(IV), pch=c(3, 17, 19),
       col=c("red", "green", "blue"))
abline(iCeptSSRI, slopeAll, col="red")
abline(iCeptPlac, slopeAll, col="green")
abline(iCeptWL,   slopeAll, col="blue")

summary(lm(DVpost ~ IV + DVpre + IV:DVpre, data=dfAncova))

##### 8.8.2
## fr Daten des Buches diese Datei laden:
# load("data_ancovaBoxplot.Rdata")
cntrMat <- rbind("SSRI-Placebo"  = c(-1,  1, 0),
                 "SSRI-WL"       = c(-1,  0, 1),
                 "Placebo-WL"    = c( 0, -1, 1),
                 "SSRI-0.5(P+WL)"= c(-2,  1, 1))
library(multcomp)
aovAncova <- aov(DVpost ~ IV + DVpre, data=dfAncova)
summary(glht(aovAncova, linfct=mcp(IV=cntrMat), alternative="greater"),
        test=adjusted("none"))

lenSqs     <- (1/nSubj) * cntrMat^2 %*% (rep(1, ncol(cntrMat)))
alphaAdj   <- 0.05/nrow(cntrMat)
(YMsAdj    <- bFull*mean(X) + aFull)
psiHats    <-  cntrMat %*% YMsAdj
fracs      <- (cntrMat %*% XMs)^2 / sum((X-ave(X, IV, FUN=mean))^2)
varPsiHats <- MSEfull * (lenSqs + fracs)
sqrt(varPsiHats)
statTs     <- psiHats / sqrt(varPsiHats)
critTs     <- rep(qt(1-alphaAdj, dfSSEfull), nrow(cntrMat))
pValTs     <- 1-pt(abs(statTs), dfSSEfull)
(resDf     <- data.frame(statTs, critTs, pVals=pValTs,
                         alphaAdj=rep(alphaAdj, nrow(cntrMat)),
                         sig=abs(statTs)>critTs))

dfSSgrp <- P-1
statFs  <- psiHats^2 / varPsiHats
critFs  <- rep(dfSSgrp*qf(1-0.05, dfSSgrp, dfSSEfull), nrow(cntrMat))
pValFs  <- 1-pf(statFs/dfSSgrp, dfSSgrp, dfSSEfull)
(resDf  <- data.frame(statFs, critFs, pVals=pValFs,
                      alpha=rep(0.05, nrow(cntrMat)), sig=statFs>critFs))

#####---------------------------------------------------------------------------
##### 8.9 Berechnung von Power und Stichprobengre
##### 8.9.1
N      <- 7
pH0    <- 0.25
pH1    <- 0.7
alpha  <- 0.05
(critB <- qbinom(1-alpha, N, pH0))
(powB  <- 1-pbinom(critB, N, pH1))
sum(dbinom(5:7, N, pH1))

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.lab=1.2, cex.main=1.2)
dH0 <- dbinom(0:N, N, pH0)
dH1 <- dbinom(0:N, N, pH1)
mat <- rbind(dH0, dH1)
rownames(mat) <- c("H0 (p=0.25)", "H1 (p=0.7)")
colnames(mat) <- 0:N
barsX <- barplot(mat, beside=TRUE, ylim=c(0, 0.35), xlab="Anzahl Treffer",
                 ylab="Wahrscheinlichkeit",
                 main="Binomialverteilungen unter H0 und H1 mit N=7",
                 col=c(rgb(1, 0.2, 0.2, 0.7), rgb(0.3, 0.3, 1, 0.6)),
                 names.arg=colnames(mat), legend.text=rownames(mat))
barplot(mat[, 1:(critB+1)], beside=TRUE, ylim=c(0, 0.35),
        col=c("red", "blue"), add=TRUE)
xx <- barsX[2, critB+1] + (barsX[1, critB+2] - barsX[2, critB+1])/4
abline(v=xx, col="green", lwd=2)
text(xx - 0.4, 0.34, adj=1, label="kritischer Wert")

Nvec     <- 2:15
critBvec <-   qbinom(1-alpha,  size=Nvec, prob=pH0)
powBvec  <- 1-pbinom(critBvec, size=Nvec, prob=pH1)

opMar <- par(mar=c(5, 4, 4, 4))
plot(Nvec, critBvec, main="Powerfunktion und kritischer Wert beim Binomialtest",
     xlab="N", xaxt="n", yaxt="n", lwd=2, type="s", pch=16, col="red",
     ylab="kritischer Wert", cex=1.4, cex.main=1)
axis(side=1, at=seq(Nvec[1], Nvec[length(Nvec)], by=1))
axis(side=2, at=seq(min(critBvec), max(critBvec), by=1), col="red")
par(new=TRUE)
plot(Nvec, powBvec, ylim=c(0, 1), type="b", lwd=2, pch=16, col="blue",
     xlab=NA, ylab=NA, axes=FALSE, cex=1.4)
axis(side=4, at=seq(0, 1, by=0.1), col="blue")
mtext(text="Power", side=4, line=3, cex=1.4)
par(opMar)
par(op)

##### 8.9.2
nSubj  <- 10
muH0   <- 0
muH1   <- 1.6
alpha  <- 0.05
sigma  <- 2

(d     <- (muH1-muH0) / sigma)
(delta <- (muH1-muH0) / (sigma/sqrt(nSubj)))
(critT <- qt(1-alpha, nSubj-1))
(powT  <- 1-pt(critT, nSubj-1, delta))

tVals  <- seq(-5, 10, length.out=100)
yH0    <- dt(tVals, nSubj-1, 0)
yH1    <- dt(tVals, nSubj-1, delta)
dev.new(width=7, height=7)
plot(tVals, yH0, type="l", lwd=2, col="red", xlab="t", ylab="Dichte",
     main="Verteilung von t unter H0 und H1", ylim=c(0, 0.4), cex.lab=1.4)
points(tVals, yH1, type="l", lwd=2, col="blue", lty=2)

tLeft  <- seq(-5,    critT, length.out=100)
tRight <- seq(critT, 10,    length.out=100)
yH0r   <- dt(tRight, nSubj-1, 0)
yH1l   <- dt(tLeft,  nSubj-1, delta)
yH1r   <- dt(tRight, nSubj-1, delta)
polygon(c(tRight, rev(tRight)), c(yH0r, numeric(length(tRight))), border=NA,
        col=rgb(1, 0.3, 0.3, 0.6))
polygon(c(tLeft,  rev(tLeft)),  c(yH1l, numeric(length(tLeft))),  border=NA,
        col=rgb(0.3, 0.3, 1, 0.6))
polygon(c(tRight, rev(tRight)), c(yH1r, numeric(length(tRight))), border=NA,
        density=5, lty=2, angle=45, col="darkgray")
abline(v=critT, lty=1, lwd=2, col=rgb(0, 1, 0, 0.5))
text(critT+0.2, 0.4,  adj=0, label="kritischer Wert")
text(critT-2.8, 0.3,  adj=1, label="Verteilung unter H0")
text(critT+1.5, 0.3,  adj=0, label="Verteilung unter H1")
text(critT+1.0, 0.08, adj=0, label="Power")
text(critT-0.7, 0.05,  expression(beta))
text(critT+0.5, 0.015, expression(alpha))

power.t.test(n=NULL, delta=muH1-muH0, sd=sigma, sig.level=0.05, power=0.9,
             type="one.sample", alternative="one.sided")

##### 8.9.3
mus   <- c(100, 110, 115)
sigma <- 10
power.anova.test(groups=3, n=NULL, between.var=var(mus), within.var=sigma^2,
                 sig.level=0.05, power=0.9)

P       <- 3
nSubj   <- c(21, 17, 19)
mus     <- c(100, 110, 115)
sigma   <- 15
grandMu <- sum(nSubj * mus) / sum(nSubj)
alphas  <- mus - grandMu
varMus  <- sum(nSubj * alphas^2) / sum(nSubj)

(fSq    <- varMus /  sigma^2)
(etaSq  <- varMus / (sigma^2 + varMus))
(lambda <- sum(nSubj * alphas^2) / sigma^2)
(critF  <- qf(1-0.05, P-1, sum(nSubj)-P))
(powF   <- 1-pf(critF, P-1, sum(nSubj)-P, lambda))

fVals   <- seq(0, 1.2, length.out=100)
nn      <- seq(10, 25, by=5)

getFPow <- function(n) {
    critF <- qf(1-0.05, P-1, P*n - P)
    1-pf(critF, P-1, P*n - P, P*n * fVals^2)
}

powsF <- sapply(nn, getFPow)

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.lab=1.2, cex.main=1.2)
yStr <- "Wahrscheinlichkeit der Annahme von H1"
mStr <- substitute(paste("Power als Funktion der Effektstrke f (", alpha, "=0.05)"))
matplot(fVals, powsF, type="l", lty=1, lwd=2, xlab="f", ylab=yStr,
        xlim=c(-0.05, 1.1), main=mStr, col=c("blue", "red", "darkgreen", "green"))
lines(c(-2, 0, 0), c(0.05, 0.05, -2), lty=2, lwd=2)
abline(h=1, lty=2, lwd=2)
mtext("f=0", side=1, at=0)
mtext(substitute(paste(alpha, "=0.05", sep="")), side=2, at=0.05, las=1)
legend(x="bottomright", legend=paste("N =", c(10, 15, 20, 25)), lwd=2,
       col=c("blue", "red", "darkgreen", "green"))

getOneFn <- function(pp, varB) {
    res <- power.anova.test(groups=P, n=NULL, between.var=varB,
                            within.var=sigma^2, sig.level=0.05, power=pp)
    res$n
}

getManyFn <- function(ff, powF) {
    varB <- ff^2 * (P/(P-1)) * sigma^2
    sapply(powF, getOneFn, varB)
}

fVals <- seq(0.1, 0.85, length.out=100)
pows  <- seq(0.8, 0.95, by=0.05)
minN  <- sapply(fVals, getManyFn, pows)

yStr  <- "Mindeststichprobengre (pro Gruppe)"
mStr  <- substitute(paste("Mindest-N als Funktion der Effektstrke f (", alpha, "=0.05)"))
matplot(fVals, t(minN), type="l", lty=1, lwd=2, xlab="f", ylab=yStr,
        xlim=c(0.1, 0.8), main=mStr, col=c("blue", "red", "darkgreen", "green"))
legend(x="topright", legend=paste("Power = ", c(0.80, 0.85, 0.90, 0.95)), lwd=2,
       col=c("blue", "red", "darkgreen", "green"))
par(op)

